package sun.misc;

import java.util.Enumeration;
import java.security.AccessController;
import java.io.FilePermission;
import java.io.IOException;
import java.security.PrivilegedAction;
import java.util.PropertyPermission;
import sun.security.util.SecurityConstants;
import java.security.Permission;
import java.net.URL;
import java.security.Permissions;
import java.io.File;
import java.security.PermissionCollection;

class PathPermissions extends PermissionCollection
{
    private static final long serialVersionUID = 8133287259134945693L;
    private File[] path;
    private Permissions perms;
    URL codeBase;
    
    PathPermissions(File[] path) {
        this.path = path;
        this.perms = null;
        this.codeBase = null;
    }
    
    URL getCodeBase() {
        return this.codeBase;
    }
    
    @Override
    public void add(Permission permission) {
        throw new SecurityException("attempt to add a permission");
    }
    
    private synchronized void init() {
        if (this.perms != null) {
            return;
        }
        (this.perms = new Permissions()).add(SecurityConstants.CREATE_CLASSLOADER_PERMISSION);
        this.perms.add(new PropertyPermission("java.*", "read"));
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                for (int i = 0; i < PathPermissions.this.path.length; ++i) {
                    File file = PathPermissions.this.path[i];
                    String str;
                    try {
                        str = file.getCanonicalPath();
                    }
                    catch (IOException ex) {
                        str = file.getAbsolutePath();
                    }
                    if (i == 0) {
                        PathPermissions.this.codeBase = Launcher.getFileURL(new File(str));
                    }
                    if (file.isDirectory()) {
                        if (str.endsWith(File.separator)) {
                            PathPermissions.this.perms.add(new FilePermission(str + "-", "read"));
                        }
                        else {
                            PathPermissions.this.perms.add(new FilePermission(str + File.separator + "-", "read"));
                        }
                    }
                    else {
                        int lastIndex = str.lastIndexOf((int)File.separatorChar);
                        if (lastIndex != -1) {
                            PathPermissions.this.perms.add(new FilePermission(str.substring(0, lastIndex + 1) + "-", "read"));
                        }
                    }
                }
                return null;
            }
        });
        return;
    }
    
    @Override
    public boolean implies(Permission permission) {
        if (this.perms == null) {
            this.init();
        }
        return this.perms.implies(permission);
    }
    
    @Override
    public Enumeration<Permission> elements() {
        if (this.perms == null) {
            this.init();
        }
        synchronized (this.perms) {
            return this.perms.elements();
        }
    }
    
    @Override
    public String toString() {
        if (this.perms == null) {
            this.init();
        }
        return this.perms.toString();
    }
}
